/*
** ===================================================================
** Name:		   : DS18B20.c
** Describe:       : Source file for basic control temperature sensor Dallas DS18B20
** Author:         : Unknown,
**                 : rewrote and remade Vt Svadbk (vitsvadbik@email.cz) 
**		   		   : BuB, March 2011
** ===================================================================
*/

#include "therm_ds18b20.h"

////////////////// initilization / reset //////////////////

uint8_t therm_reset(void)
{	
uint8_t i;	//uint8_t - data type defined in stdint.h - 8-bit signed type

THERM_LOW(); 
THERM_OUTPUT_MODE();	
_delay_us(480);

//Release line and wait for 60uS
THERM_INPUT_MODE();		
_delay_us(70);

//Store line value and wait until the completion of 480uS period

i=(THERM_PIN & (1<<THERM_DQ));	// THERM_PIN = receive level
_delay_us(410);

//Return the value read from the presence pulse (0=OK, 1=WRONG)
return i;
}

////////////////// write bit to DS18B20 //////////////////

void therm_write_bit(uint8_t bit)
{
_delay_us(1);
//Pull line low for 1uS
THERM_LOW();
THERM_OUTPUT_MODE();
_delay_us(6);

//If we want to write 1, release the line (if not will keep low)
if(bit) THERM_INPUT_MODE(); //write log. 1

//Wait for 60uS (66uS) and release the line
_delay_us(60);		//write log.0
THERM_INPUT_MODE();
}

////////////////// write byte to DS18B20 //////////////////

void therm_write_byte(uint8_t byte)
{
uint8_t i=8;

 while(i--){		//running for 7 bite 
 //Write actual bit and shift one position right to make the next bit ready
 therm_write_bit(byte&1);
 byte>>=1;
 }
}

////////////////// read bit from DS18B20 //////////////////

uint8_t therm_read_bit(void)
{
uint8_t bit=0;

//Pull line low for 1uS
THERM_LOW();
THERM_OUTPUT_MODE();
_delay_us(1);

//Release line and wait for 14uS
THERM_INPUT_MODE();
_delay_us(14);

//Read line value
if(THERM_PIN&(1<<THERM_DQ)) bit=1;	//if receive high - bit = 1

//Wait for 45uS to end and return read value
_delay_us(45);
return bit;
}

////////////////// read byte from DS18B20 //////////////////
uint8_t therm_read_byte(void)
{
uint8_t i=8, n=0;

 while(i--){	//running for 7 bite 
 //Shift one position right and store read value
 n>>=1;		
 n|=(therm_read_bit()<<7);
 }
return n;
}


